######################################################################
##
## Copyright (C) 2006,  Blekinge Institute of Technology
##
## Filename:      Objdump.py
## Author:        Simon Kagstrom <ska@bth.se>
## Description:   Objdump
##
## $Id: Objdump.py 8337 2006-05-26 20:00:53Z ska $
##
######################################################################
import re, cgi, os, dissy.architecture

from dissy.File import *
from dissy.Function import Function
from dissy.Instruction import Instruction
from dissy.Config import config

ADDRESS_REGEXP  = "[0-9,a-f,A-F]+"
ENCODING_REGEXP = ADDRESS_REGEXP + "[ ]"
FUNCTION_REGEXP = "(?:[.]*)[_,0-9,a-z,A-Z,\:,\*,\,\(,\), ,<,>]+"
INSN_REGEXP     = "[0-9,a-z,A-Z,_]+"
INSN_ARGS_REGEXP= "\**[a-z,A-Z,0-9,_,\,,\(,\),\%,\$,\[,\],!,#,\-, ]+"

class Objdump:
    def __init__(self, filename):
	self.labelDefRegexp = re.compile("(" + ADDRESS_REGEXP + ") <(" + FUNCTION_REGEXP + ")>:\n")
	self.insnRegExp = re.compile("[ ]*(" + ADDRESS_REGEXP + "):[ \t]+((?:" + ENCODING_REGEXP +")*)[ \t]+(" + INSN_REGEXP + ")+[ \t]*(" + INSN_ARGS_REGEXP + ")*")
	self.filename = filename
	self.stream = os.popen(config.objdumpCommand % self.filename)
	self.arch = dissy.architecture.getArchitecture(self.getArchStr())
	self.outFile = File(self.arch)
	self.curFn = None

    def getArchStr(self):
	"Get the architecture of the file"
	arch = "intel" # Assume Intel architecture
	f = os.popen(config.readelfCommand % self.filename)
	for line in f:
	    words = line.split()
	    if len(words) >= 2 and words[0] == "Machine:":
		arch = words[1].lower()
	f.close()
	return arch

    def parse(self, fnCount = None):
	i = 0

	# Current function
	fn = self.curFn
	for line in self.stream:
	    r = self.labelDefRegexp.match(line)
	    if r != None:
		label = cgi.escape(r.group(2))

		fn = Function(self.outFile, long("0x" + r.group(1),16), label)
		self.outFile.addFunction(fn)
		i = i + 1
		if fnCount and i > fnCount:
		    # Stop early (then this is restartable)
		    self.curFn = fn
		    return self.outFile, False
		continue
	    if fn != None:
		r = self.insnRegExp.match(line)
		if r != None:
		    insn = Instruction(fn, long("0x" + r.group(1),16), r.group(2), r.group(3), r.group(4))
		    fn.addInstruction(insn)
		else:
		    fn.addOther(cgi.escape(line))
	self.stream.close()
	return self.outFile, True

if __name__ == "__main__":
    objdump = Objdump("/dev/stdin")
    print objdump.parse()
